package Onis::Users;

use strict;
use warnings;
use Exporter;
use Onis::Config qw#get_config#;
use Onis::Data::Persistent qw#init#;

@Onis::Users::EXPORT_OK = qw#host_to_username nick_to_username get_link get_image get_name#;
@Onis::Users::ISA = ('Exporter');

=head1 Onis/Users.pm

Parses user-info and provides query-routines.

=head1 Usage

use Onis::Users qw#host_to_username get_link get_image#;

$username = host_to_username ($hostname);
$username = nick_to_username ($nick);

my $name  = get_name  ($username);
my $link  = get_link  ($username);
my $image = get_image ($username);

=head1 Diagnostics

Set $::DEBUG to ``0x1000'' to get extra debug messages.

=cut

our $Users = {};
our $HostmaskCache = init ('$HostmaskCache', 'hash');
our $NicknameCache = init ('$NicknameCache', 'hash');

my $VERSION = '$Id: Users.pm,v 1.2 2004/08/01 13:45:27 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

read_config ();

return (1);

=head2 read_config ()

Reads the configuration using Onis::Config::get_config ()

=cut

sub read_config
{
	my $config_file = 'users.conf';
	my $content;
	my $fh;
	
	if (get_config ('users_config'))
	{
		my $temp = get_config ('users_config');
		if (-e $temp and -r $temp)
		{
			$config_file = $temp;
		}
		elsif (-e $temp)
		{
			print STDERR $/, __FILE__, ": Unable to read users_config ``$temp'': ",
				"File not readable. Check your permissions.";
		}
		else
		{
			print STDERR $/, __FILE__, ": Unable to read users_config ``$temp'': ",
				"File does not exist.";
		}
	}

	# Fail silently, if fle does not exist..
	if (!-e $config_file) { return (0); }

	print STDERR $/, __FILE__, ": Reading config file ``$config_file''" if ($::DEBUG & 0x1000);

	# read the file
	unless (open ($fh, "< $config_file"))
	{
		print STDERR $/, __FILE__, ": Unable to open ``$config_file'' for reading: $!";
		return (0);
	}

	{
		local ($/) = undef;
		$content = <$fh>;
	}

	close ($fh);

	# parse the file
	#$content =~ s/[\n\r\s]+//gs;
	$content =~ s/#.*$//gm;
	$content =~ s/[\n\r]+//gs;
	
	#while ($content =~ m/([^{]+){([^}]+)}/g)
	while ($content =~ m/([^\s{]+)\s*{([^}]+)}/g)
	{
		my $user = $1;
		my $line = $2;

		print STDERR $/, __FILE__, ": User ``$user''" if ($::DEBUG & 0x1000);

		while ($line =~ m/([^\s:]+)\s*:([^;]+);/g)
		{
			my $key = lc ($1);
			my $val = $2;
			$val =~ s/^\s+|\s+$//g;

			print STDERR $/, __FILE__, ": + $key = ``$val''" if ($::DEBUG & 0x1000);

			if (($key eq 'image') or ($key eq 'link')
					or ($key eq 'name'))
			{
				if (!defined ($Users->{$user}{$key}))
				{
					$Users->{$user}{$key} = [];
				}
				push (@{$Users->{$user}{$key}}, $val);
			}
			elsif (($key eq 'host') or ($key eq 'hostmask'))
			{
				my $this_nick;
				my $this_user;
				my $this_host;

				if ($val =~ m/^([^!]+)!([^@]+)@(.+)$/)
				{
					$this_nick = quotemeta (lc ($1));
					$this_user = quotemeta (lc ($2));
					$this_host = quotemeta (lc ($3));
				}
				else
				{
					print STDERR $/, __FILE__, ": Invalid hostmask for user $user: ``$val''";
					next;
				}

				$this_nick =~ s/\\\*/[^!]*/g;
				$this_nick =~ s/\\\?/[^!]/g;

				$this_user =~ s/\\\*/[^@]*/g;
				$this_user =~ s/\\\?/[^@]/g;

				$this_host =~ s/\\\*/.*/g;
				$this_host =~ s/\\\?/./g;

				$val = "$this_nick!$this_user\@$this_host";

				if (!defined ($Users->{$user}{'host'}))
				{
					$Users->{$user}{'host'} = [];
				}

				print STDERR " --> m/^$val\$/i" if ($::DEBUG & 0x1000);
				
				push (@{$Users->{$user}{'host'}}, qr/^$val$/i);
			}
			else
			{
				print STDERR $/, __FILE__, ": Invalid key in users_config: ``$key''";
			}
		}

		if (!defined ($Users->{$user}{'host'}))
		{
			print STDERR $/, __FILE__, ": No hostmask given for user $user. Ignoring him/her.";
			delete ($Users->{$user});
		}
	}

	return (1);
}

=head2 host_to_username ($hostname)

Matches the hostname against the configured hostmasks. Uses caching to
speed up execution.

=cut

sub host_to_username
{
	my $host = shift;
	my $user = '';
	my $nick;

	if ($host =~ m/^([^!]+)/)
	{
		$nick = $1;
		$host =~ s/![\~\^\-\+\=]/!/;
	}
	else
	{
		# FIXME Shoudl we yell at someone?
		die;
		return ('');
	}

	if (defined ($HostmaskCache->{$host}))
	{
		$user = $HostmaskCache->{$host};
	}
	else
	{
		USER: for (keys (%$Users))
		{
			my $this_user = $_;
			for (@{$Users->{$this_user}{'host'}})
			{
				my $host_re = $_;

				if ($host =~ $host_re)
				{
					$user = $this_user;
					$NicknameCache->{$nick} = $user;
					$NicknameCache->{$user} = $user;
					last (USER);
				}
			}
		}

		if (($::DEBUG & 0x1000) and $user)
		{
			print STDERR $/, __FILE__, ": Host ``$host'' belongs to ``$user''";
		}
	}
	
	$HostmaskCache->{$host} = $user;
	return ($user);
}

sub nick_to_username
{
	my $nick = shift;

	if (defined ($NicknameCache->{$nick}))
	{
		return ($NicknameCache->{$nick});
	}
	else
	{
		return ('');
	}
}

sub get_name
{
	my $user = shift;
	my $retval = '';

	if (defined ($Users->{$user}{'name'}))
	{
		my $tmp = int (rand (scalar (@{$Users->{$user}{'name'}})));
		$retval = $Users->{$user}{'name'}[$tmp];
	}

	return ($retval);
}

=head2 get_link ($user)

Returns the URL of the users link, if one is configured.

=cut

sub get_link
{
	my $user = shift;
	my $retval = '';

	if (defined ($Users->{$user}{'link'}))
	{
		my $tmp = int (rand (scalar (@{$Users->{$user}{'link'}})));
		$retval = $Users->{$user}{'link'}[$tmp];
	}

	return ($retval);
}

=head2 get_image ($user)

Returns the URL of the users image, if one is configured.

=cut

sub get_image
{
	my $user = shift;
	my $retval = '';

	if (defined ($Users->{$user}{'image'}))
	{
		my $tmp = int (rand (scalar (@{$Users->{$user}{'image'}})));
		$retval = $Users->{$user}{'image'}[$tmp];
	}

	return ($retval);
}

=head1 Author

Florian octo Forster E<lt>octo@verplant.orgE<gt>
